@@ -4,7 +4,10 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite,
 		   int *nObs, double *locs, double *scales, double *shapes,
 		   double *sill, double *range, double *smooth,
 		   int *fitmarge,double *dns){
-  //This is the schlater model. It computes the pairwise log-likelihood
+  //This is the schlather's model. It's a wrapper to several
+  //sub-functions. It's named xxxfull as it either assume that the
+  //margins are unit Frechet, or the GEV parameters are estimated at
+  //each locations.
   
   const int nPairs = *nSite * (*nSite - 1) / 2;
   int i;
@@ -14,17 +17,20 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite,
   rho = (double *)R_alloc(nPairs, sizeof(double));
   frech = (double *)R_alloc(*nSite * *nObs, sizeof(double));
 
+  *dns = 1.0;
   //Some preliminary steps: Valid points?
   if (*fitmarge){
     for (i=0;i<*nSite;i++){
       if (scales[i] <= 0){
 	//printf("scales <= 0!!!\n");
-	*dns += R_pow_di(1 - scales[i], 2) * MINF;
+	*dns += R_pow_di(1 - scales[i], 2);
+	scales[i] = 1.0;
       }
       
       if (shapes[i] <= -1){
 	//printf("shapes <= -1!!!\n");
-	*dns += R_pow_di(shapes[i], 2) * MINF;
+	*dns += R_pow_di(shapes[i], 2);
+	shapes[i] = 0.0;
       }
     }
   }
@@ -53,12 +59,12 @@ void schlatherfull(int *covmod, double *data, double *dist, int *nSite,
       jac[i] = 0.0;
     }
   }
-  
-  if (*dns == 0.0){
-    //Stage 2: Bivariate density computations
-    *dns = lplikschlather(frech, rho, jac, *nObs, *nSite);
-  }  
 
+  *dns *= lplikschlather(frech, rho, jac, *nObs, *nSite);
+
+  if (!R_FINITE(*dns))
+    *dns = MINF;
+  
   return;
 
 }
@@ -70,7 +76,7 @@ void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
 		      double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
 		      double *loccoeff, double *scalecoeff, double *shapecoeff, double *sill,
 		      double *range, double *smooth, double *dns){
-  //This is the schlater model
+  //This is the Schlather's model.
   //The GEV parameters are defined using a polynomial response surface
   
   const int nPairs = *nSite * (*nSite - 1) / 2;
@@ -83,56 +89,51 @@ void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
   shapes = (double *)R_alloc(*nSite, sizeof(double));
   frech = (double *)R_alloc(*nObs * *nSite, sizeof(double));
   
+  *dns = 1.0;
   //Stage 1: Compute the covariance at each location
   switch (*covmod){
   case 1:
-    *dns = whittleMatern(dist, nPairs, *sill, *range, *smooth, rho);
+    *dns += whittleMatern(dist, nPairs, *sill, *range, *smooth, rho);
     break;
   case 2:
-    *dns = cauchy(dist, nPairs, *sill, *range, *smooth, rho);
+    *dns += cauchy(dist, nPairs, *sill, *range, *smooth, rho);
     break;
   case 3:
-    *dns = powerExp(dist, nPairs, *sill, *range, *smooth, rho);
+    *dns += powerExp(dist, nPairs, *sill, *range, *smooth, rho);
     break;
   }
-  
-  if (*dns != 0.0){
-    //printf("problem with covariance param.\n");
-    return;
-  }
 
   //Stage 2: Compute the GEV parameters using the design matrix
   *dns += dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat,
-			loccoeff, scalecoeff, shapecoeff, *nSite,
-			*nloccoeff, *nscalecoeff, *nshapecoeff,
-			locs, scales, shapes);
+		      loccoeff, scalecoeff, shapecoeff, *nSite,
+		      *nloccoeff, *nscalecoeff, *nshapecoeff,
+		      locs, scales, shapes);
 
   //Stage 3: Transformation to unit Frechet
   *dns += gev2frech(data, *nObs, *nSite, locs, scales, shapes,
 		    jac, frech);
-    
-  if (*dns == 0.0){
 
-    //Stage 4: Bivariate density computations
-    *dns = lplikschlather(frech, rho, jac, *nObs, *nSite);
-    
-    //Stage 5: Removing the penalizing terms (if any)
-    // 1- For the location parameter
-    if (*locpenalty > 0)
-      *dns -= penalization(locpenmat, loccoeff, *locpenalty,
-			   *nloccoeff, *npparloc);
-    
-    // 2- For the scale parameter
-    if (*scalepenalty > 0)    
-      *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty,
-			   *nscalecoeff, *npparscale);
-    
-    // 3- For the shape parameter
-    if (*shapepenalty > 0)
-      *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty,
-			   *nshapecoeff, *npparshape);
-  }
+  *dns *= lplikschlather(frech, rho, jac, *nObs, *nSite);
+
+  //Stage 5: Removing the penalizing terms (if any)
+  // 1- For the location parameter
+  if (*locpenalty > 0)
+    *dns -= penalization(locpenmat, loccoeff, *locpenalty,
+			 *nloccoeff, *npparloc);
   
+  // 2- For the scale parameter
+  if (*scalepenalty > 0)    
+    *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty,
+			 *nscalecoeff, *npparscale);
+  
+  // 3- For the shape parameter
+  if (*shapepenalty > 0)
+    *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty,
+			 *nshapecoeff, *npparshape);
+
+  if (!R_FINITE(*dns))
+    *dns = MINF;
+
   return;
   
 }
