@@ -4,7 +4,10 @@ void smithfull(double *data, double *distVec, int *nSite,
 	       int *nObs, double *locs, double *scales, double *shapes,
 	       double *cov11, double *cov12, double *cov22,
 	       int *fitmarge, double *dns){
-  //This is the Smith model. It computes the pairwise log-likelihood
+  //This is the Smith's model. It's a wrapper to several
+  //sub-functions. It's named xxxfull as it either assume that the
+  //margins are unit Frechet, or the GEV parameters are estimated at
+  //each locations.
 
   const int nPairs = *nSite * (*nSite - 1) / 2;
   int i;
@@ -14,17 +17,20 @@ void smithfull(double *data, double *distVec, int *nSite,
   mahalDist = (double *)R_alloc(nPairs, sizeof(double));
   frech = (double *)R_alloc(*nSite * *nObs, sizeof(double));
   
+  *dns = 1.0;
   //Some preliminary steps: Valid points?
   if (*fitmarge){
     for (i=0;i<*nSite;i++){
       if (scales[i] <= 0){
 	//printf("scales <= 0!!!\n");
-	*dns += R_pow_di(1 - scales[i], 2) * MINF;
+	*dns += R_pow_di(1 - scales[i], 2);
+	scales[i] = .1;
       }
       
       if (shapes[i] <= -1){
 	//printf("shapes <= -1!!!\n");
-	*dns += R_pow_di(shapes[i], 2) * MINF;
+	*dns += R_pow_di(shapes[i], 2);
+	shapes[i] = 0.0;
       }
     }
   }
@@ -44,10 +50,10 @@ void smithfull(double *data, double *distVec, int *nSite,
     }
   }
   
-  if (*dns == 0.0){
-    //Stage 3: Bivariate density computations
-    *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
-  }  
+  *dns *= lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
+
+  if (!R_FINITE(*dns))
+    *dns = MINF;
 
   return;
 }
@@ -60,7 +66,8 @@ void smithdsgnmat(double *data, double *distVec, int *nSite, int *nObs,
 		  int *nshapecoeff, int *npparshape, double *shapepenalty,
 		  double *loccoeff, double *scalecoeff, double *shapecoeff,
 		  double *cov11, double *cov12, double *cov22, double *dns){
-  //This is the Smith model. It computes the pairwise log-likelihood
+  //This is the Smith's model. It's named xxxdsgnmat as either linear
+  //models or p-splines are used for the gev parameters.
   
   const int nPairs = *nSite * (*nSite - 1) / 2;
   double *jac, *mahalDist, *locs, *scales, *shapes, *frech;
@@ -72,40 +79,41 @@ void smithdsgnmat(double *data, double *distVec, int *nSite, int *nObs,
   shapes = (double *)R_alloc(*nSite, sizeof(double));
   frech = (double *)R_alloc(*nSite * *nObs, sizeof(double));
   
+  *dns = 1.0;
   //Stage 1: Computing the Mahalanobis distance
-  *dns = mahalDistFct(distVec, nPairs, cov11, cov12,
-		      cov22, mahalDist);
+  *dns += mahalDistFct(distVec, nPairs, cov11, cov12,
+		       cov22, mahalDist);
 
   //Stage 2: Computing the GEV parameters using the design matrix
   *dns += dsgnmat2Param(locdsgnmat, scaledsgnmat, shapedsgnmat,
 			loccoeff, scalecoeff, shapecoeff, *nSite,
 			*nloccoeff, *nscalecoeff, *nshapecoeff,
 			locs, scales, shapes);
-  
+
   //Stage 3: Transformation to unit Frechet
   *dns += gev2frech(data, *nObs, *nSite, locs, scales, shapes,
 		    jac, frech);
+  
+  *dns *= lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
     
-  if (*dns == 0.0){
-    //Stage 4: Bivariate density computations
-    *dns = lpliksmith(frech, mahalDist, jac, *nObs, *nSite);
-    
-    //Stage 5: Removing the penalizing terms (if any)
-    // 1- For the location parameter
-    if (*locpenalty > 0)
-      *dns -= penalization(locpenmat, loccoeff, *locpenalty,
-			   *nloccoeff, *npparloc);
-    
-    // 2- For the scale parameter
-    if (*scalepenalty > 0)    
-      *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty,
-			   *nscalecoeff, *npparscale);
-    
-    // 3- For the shape parameter
-    if (*shapepenalty > 0)
-      *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty,
-			   *nshapecoeff, *npparshape);
-  }
+  //Stage 5: Removing the penalizing terms (if any)
+  // 1- For the location parameter
+  if (*locpenalty > 0)
+    *dns -= penalization(locpenmat, loccoeff, *locpenalty,
+			 *nloccoeff, *npparloc);
+  
+  // 2- For the scale parameter
+  if (*scalepenalty > 0)    
+    *dns -= penalization(scalepenmat, scalecoeff, *scalepenalty,
+			 *nscalecoeff, *npparscale);
+  
+  // 3- For the shape parameter
+  if (*shapepenalty > 0)
+    *dns -= penalization(shapepenmat, shapecoeff, *shapepenalty,
+			 *nshapecoeff, *npparshape);
+
+  if (!R_FINITE(*dns))
+    *dns = MINF;
   
   return;
 }
